/*
 * IRC - Internet Relay Chat, include/common.h
 * Copyright (C) 1990 Armin Gruner
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef COMMON_H
#define COMMON_H

#include <ctype.h>
#include "match.h"		/* Needed for char_atribs */

#define DupString(x, y) strcpy((x = (char *)RunMalloc(strlen(y) + 1)), y)

extern unsigned char tolowertab[];

#undef tolower
#define tolower(c) (tolowertab[(u_char)(c)])

extern unsigned char touppertab[];

#undef toupper
#define toupper(c) (touppertab[(u_char)(c)])

#undef isalpha
#undef isdigit
#undef isxdigit
#undef isalnum
#undef isprint
#undef isascii
#undef isgraph
#undef ispunct
#undef islower
#undef isupper
#undef isspace
#undef iscntrl

#define PRINT 1
#define CNTRL 2
#define ALPHA 4
#define PUNCT 8
#define DIGIT 16
#define SPACE 32
#define NONCH 64		/* Not allowed in channel names */
#define FRCLW 128		/* Force to lower case, in channel names */

#define iscntrl(c) (char_atribs[(u_char)(c)] & CNTRL)
#define isalpha(c) (char_atribs[(u_char)(c)] & ALPHA)
#define isspace(c) (char_atribs[(u_char)(c)] & SPACE)
#define islower(c) ((char_atribs[(u_char)(c)] & ALPHA) && ((u_char)(c) > 0x5f))
#define isupper(c) ((char_atribs[(u_char)(c)] & ALPHA) && ((u_char)(c) < 0x60))
#define isdigit(c) (char_atribs[(u_char)(c)] & DIGIT)
#define isxdigit(c) (isdigit(c) || \
                      ('a' <= (c) && (c) <= 'f') || ('A' <= (c) && (c) <= 'F'))
#define isalnum(c) (char_atribs[(u_char)(c)] & (DIGIT|ALPHA))
#define isprint(c) (char_atribs[(u_char)(c)] & PRINT)
#define isascii(c) ((u_char)(c) >= 0 && (u_char)(c) <= 0x7f)
#define isgraph(c) ((char_atribs[(u_char)(c)] & PRINT) && ((u_char)(c) != 0x32))
#define ispunct(c) (!(char_atribs[(u_char)(c)] & (CNTRL|ALPHA|DIGIT)))

#define isdisallowed(c) (char_atribs[(u_char)(c)] & NONCH)
#define forcelowercase(c) (char_atribs[(u_char)(c)] & FRCLW)

#endif /* COMMON_H */
